@@ -107,7 +107,7 @@
 #define  INSPIRALTEMPLATE_SOURCEPHI             0.
 #define  INSPIRALTEMPLATE_POLARISATIONANGLE     0.
 
-#define INSPIRALTEMPLATE_INTERACTION	   LAL_SIM_INSPIRAL_INTERACTION_ALL_SPIN
+#define INSPIRALTEMPLATE_INTERACTION	   LAL_INSPIRAL_INTERACTION_ALL_SPIN
 #define INSPIRALTEMPLATE_AXISCHOICE             LAL_SIM_INSPIRAL_FRAME_AXIS_VIEW
 #define INSPIRALTEMPLATE_FIXEDSTEP              0
 #define INSPIRALTEMPLATE_INSPIRALONLY           0
@@ -299,27 +299,27 @@ void LALInspiralITStructureParseParameters(LALStatus *status,
       else if (strcmp(argv[i],"--interaction")==0)
       {
         if (strcmp(argv[++i],"NO")==0){
-          params->interaction = LAL_SIM_INSPIRAL_INTERACTION_NONE; }
+          params->interaction = LAL_INSPIRAL_INTERACTION_NONE; }
         else if (strcmp(argv[i],"SO15")==0){
-          params->interaction = LAL_SIM_INSPIRAL_INTERACTION_SPIN_ORBIT_15PN; }
+          params->interaction = LAL_INSPIRAL_INTERACTION_SPIN_ORBIT_15PN; }
         else if (strcmp(argv[i],"SS")==0){
-          params->interaction = LAL_SIM_INSPIRAL_INTERACTION_SPIN_SPIN_2PN; }
+          params->interaction = LAL_INSPIRAL_INTERACTION_SPIN_SPIN_2PN; }
         else if (strcmp(argv[i],"SSself")==0){
-          params->interaction = LAL_SIM_INSPIRAL_INTERACTION_SPIN_SPIN_SELF_2PN; }
+          params->interaction = LAL_INSPIRAL_INTERACTION_SPIN_SPIN_SELF_2PN; }
         else if (strcmp(argv[i],"QM")==0){
-          params->interaction = LAL_SIM_INSPIRAL_INTERACTION_QUAD_MONO_2PN; }
+          params->interaction = LAL_INSPIRAL_INTERACTION_QUAD_MONO_2PN; }
         else if (strcmp(argv[i],"SO25")==0){
-          params->interaction = LAL_SIM_INSPIRAL_INTERACTION_SPIN_ORBIT_25PN; }
+          params->interaction = LAL_INSPIRAL_INTERACTION_SPIN_ORBIT_25PN; }
         else if (strcmp(argv[i],"SO30")==0){
-          params->interaction = LAL_SIM_INSPIRAL_INTERACTION_SPIN_ORBIT_3PN; }
+          params->interaction = LAL_INSPIRAL_INTERACTION_SPIN_ORBIT_3PN; }
         else if (strcmp(argv[i],"AllSpin")==0){
-          params->interaction = LAL_SIM_INSPIRAL_INTERACTION_ALL_SPIN; }
+          params->interaction = LAL_INSPIRAL_INTERACTION_ALL_SPIN; }
         else if (strcmp(argv[i],"Tidal")==0){
-          params->interaction = LAL_SIM_INSPIRAL_INTERACTION_TIDAL_5PN; }
+          params->interaction = LAL_INSPIRAL_INTERACTION_TIDAL_5PN; }
         else if (strcmp(argv[i],"Tidal6")==0){
-          params->interaction = LAL_SIM_INSPIRAL_INTERACTION_TIDAL_6PN; }
+          params->interaction = LAL_INSPIRAL_INTERACTION_TIDAL_6PN; }
         else if (strcmp(argv[i],"All")==0){
-          params->interaction = LAL_SIM_INSPIRAL_INTERACTION_ALL; }
+          params->interaction = LAL_INSPIRAL_INTERACTION_ALL; }
         else
           fprintf(stderr,"Invalid choice of --interaction\n");
       }
